#include "hw_impl_zybo.h"

#define THROTTLE_PULSE_WIDTH_LOW 1000 // us
#define THROTTLE_PULSE_WIDTH_HIGH 2000 // us
#define PERIOD_WIDTH 222222 // 100000000 ns / 450
#define PULSE_WIDTH_ADDR_OFFSET 4

struct PWMOutputDriverState {
  int *outputs[4];
};

int zybo_pwm_output_reset(struct PWMOutputDriver *self) {
  if (self->state == NULL) {
    self->state = malloc(sizeof(struct PWMOutputDriverState));
    if (self->state == NULL) {
      return -1;
    }
  }
  struct PWMOutputDriverState *state = self->state;

  state->outputs[0] = (int *) XPAR_PWM_SIGNAL_OUT_WKILLSWITCH_0_BASEADDR;
  state->outputs[1] = (int *) XPAR_PWM_SIGNAL_OUT_WKILLSWITCH_1_BASEADDR;
  state->outputs[2] = (int *) XPAR_PWM_SIGNAL_OUT_WKILLSWITCH_2_BASEADDR;
  state->outputs[3] = (int *) XPAR_PWM_SIGNAL_OUT_WKILLSWITCH_3_BASEADDR;

  // Set period width of PWM pulse
  *(state->outputs[0]) = PERIOD_WIDTH;
  *(state->outputs[1]) = PERIOD_WIDTH;
  *(state->outputs[2]) = PERIOD_WIDTH;
  *(state->outputs[3]) = PERIOD_WIDTH;

  // Set a low pulse (1 ms) so that outputs are off
  *(state->outputs[0] + PULSE_WIDTH_ADDR_OFFSET) = THROTTLE_PULSE_WIDTH_LOW;
  *(state->outputs[1] + PULSE_WIDTH_ADDR_OFFSET) = THROTTLE_PULSE_WIDTH_LOW;
  *(state->outputs[2] + PULSE_WIDTH_ADDR_OFFSET) = THROTTLE_PULSE_WIDTH_LOW;
  *(state->outputs[3] + PULSE_WIDTH_ADDR_OFFSET) = THROTTLE_PULSE_WIDTH_LOW;

  usleep(1000000);
  return 0;
}

int zybo_pwm_output_write(struct PWMOutputDriver *self,
                          unsigned int channel,
                          unsigned long pulse_width_us) {
  if (pulse_width_us > THROTTLE_PULSE_WIDTH_HIGH)
    pulse_width_us = THROTTLE_PULSE_WIDTH_HIGH;
  if (pulse_width_us < THROTTLE_PULSE_WIDTH_LOW)
    pulse_width_us = THROTTLE_PULSE_WIDTH_LOW;
  struct PWMOutputDriverState *state = self->state;
  *(state->outputs[channel] + PULSE_WIDTH_ADDR_OFFSET) = pulse_width_us;
  return 0;
}
